## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import re
import glob
from sos.plugins import Plugin


class smartmon(Plugin):
    """Dump SMARK disk information.
    """

    # Only disks have a device.  Unfortunately not all devices are
    # suitable for smartctl and some devices need different handling.
    #
    # The following table is a list of regular expressions that should
    # match the device symlink target.  If the second element of the tuple
    # is None then this is a device that should be skipped, if callable
    # then the function should return a list or tuple when called with the
    # original string and the match and otherwise should be a list or
    # tuple of strings.
    handlers = (
        ('/vbd-[0-9]*$', None),
        ('/cciss([0-9]+)/c[0-9]+d[0-9]+$', lambda s, m: ["-d", "cciss,%s" % m.group(1)])
    )

    def setup(self):
        for name in glob.glob('/sys/block/*/device'):
            f = None
            link = None
            devname = None
            try:
                link = os.readlink(name)
                f = open(os.path.join(os.path.dirname(name), 'uevent'))
                for l in f.readlines():
                    (tag, value) = l.split('=', 2)
                    if tag == 'DEVNAME':
                        devname = '/dev/' + value.strip()
                        break
            except (OSError, IOError):
                continue
            finally:
                if f:
                    f.close()
            if link is None or devname is None:
                continue

            smart = ['/usr/sbin/smartctl', '-a']
            for (rexp, handler) in self.handlers:
                m = re.search(rexp, link)
                if m:
                    if handler is None:
                        smart = None
                    elif callable(handler):
                        args = handler(link, m)
                        if args:
                            smart.extend(args)
                    else:
                        smart.extend(handler)
                    break
            if smart is not None:
                smart.append(devname)
                self.collectExtOutput(" ".join(smart))
